@@ -2,6 +2,7 @@ package main
 
 import (
 	"bytes"
+	"encoding/base64"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
@@ -11,24 +12,23 @@ import (
 	"os"
 	"os/exec"
 	"path"
-	"strconv"
 	"strings"
 	"testing"
 	"time"
 
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/git"
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/gitaly"
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/testhelper"
-
-	pb "gitlab.com/gitlab-org/gitaly-proto/go"
-
 	"github.com/golang/protobuf/jsonpb"
 	"github.com/golang/protobuf/proto"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/codes"
+
+	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/git"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/gitaly"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/testhelper"
 )
 
 func TestFailedCloneNoGitaly(t *testing.T) {
@@ -38,13 +38,12 @@ func TestFailedCloneNoGitaly(t *testing.T) {
 	authBody := &api.Response{
 		GL_ID:       "user-123",
 		GL_USERNAME: "username",
-		RepoPath:    repoPath(t),
 		// This will create a failure to connect to Gitaly
 		GitalyServer: gitaly.Server{Address: "unix:/nonexistent"},
 	}
 
 	// Prepare test server and backend
-	ts := testAuthServer(nil, 200, authBody)
+	ts := testAuthServer(nil, nil, 200, authBody)
 	defer ts.Close()
 	ws := startWorkhorseServer(ts.URL)
 	defer ws.Close()
@@ -60,34 +59,84 @@ func TestGetInfoRefsProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 
 	apiResponse := gitOkBody(t)
 	apiResponse.GitalyServer.Address = gitalyAddress
 
-	for _, showAllRefs := range []bool{true, false} {
-		t.Run(fmt.Sprintf("ShowAllRefs=%v", showAllRefs), func(t *testing.T) {
-			apiResponse.ShowAllRefs = showAllRefs
+	goodMetadata := map[string]string{
+		"gitaly-feature-foobar": "true",
+		"gitaly-feature-bazqux": "false",
+	}
+	badMetadata := map[string]string{
+		"bad-metadata": "is blocked",
+	}
 
-			ts := testAuthServer(nil, 200, apiResponse)
+	features := make(map[string]string)
+	for k, v := range goodMetadata {
+		features[k] = v
+	}
+	for k, v := range badMetadata {
+		features[k] = v
+	}
+	apiResponse.GitalyServer.Features = features
+
+	testCases := []struct {
+		showAllRefs bool
+		gitRpc      string
+	}{
+		{showAllRefs: false, gitRpc: "git-upload-pack"},
+		{showAllRefs: true, gitRpc: "git-upload-pack"},
+		{showAllRefs: false, gitRpc: "git-receive-pack"},
+		{showAllRefs: true, gitRpc: "git-receive-pack"},
+	}
+
+	for _, tc := range testCases {
+		t.Run(fmt.Sprintf("ShowAllRefs=%v,gitRpc=%v", tc.showAllRefs, tc.gitRpc), func(t *testing.T) {
+			apiResponse.ShowAllRefs = tc.showAllRefs
+
+			ts := testAuthServer(nil, nil, 200, apiResponse)
 			defer ts.Close()
 
 			ws := startWorkhorseServer(ts.URL)
 			defer ws.Close()
 
-			resource := "/gitlab-org/gitlab-test.git/info/refs?service=git-upload-pack"
-			_, body := httpGet(t, ws.URL+resource)
+			gitProtocol := "fake git protocol"
+			resource := "/gitlab-org/gitlab-test.git/info/refs?service=" + tc.gitRpc
+			resp, body := httpGet(t, ws.URL+resource, map[string]string{"Git-Protocol": gitProtocol})
 
-			expectedContent := "\n\000" + string(testhelper.GitalyInfoRefsResponseMock) + "\000"
-			if showAllRefs {
-				expectedContent = git.GitConfigShowAllRefs + expectedContent
+			require.Equal(t, 200, resp.StatusCode)
+
+			bodySplit := strings.SplitN(body, "\000", 3)
+			require.Len(t, bodySplit, 3)
+
+			gitalyRequest := &gitalypb.InfoRefsRequest{}
+			require.NoError(t, jsonpb.UnmarshalString(bodySplit[0], gitalyRequest))
+
+			require.Equal(t, gitProtocol, gitalyRequest.GitProtocol)
+			if tc.showAllRefs {
+				require.Equal(t, []string{git.GitConfigShowAllRefs}, gitalyRequest.GitConfigOptions)
+			} else {
+				require.Empty(t, gitalyRequest.GitConfigOptions)
 			}
 
-			assert.Equal(t, expectedContent, body, "GET %q: response body", resource)
+			require.Equal(t, tc.gitRpc, bodySplit[1])
 
+			require.Equal(t, string(testhelper.GitalyInfoRefsResponseMock), bodySplit[2], "GET %q: response body", resource)
+
+			md := gitalyServer.LastIncomingMetadata
+			for k, v := range goodMetadata {
+				actual := md[k]
+				require.Len(t, actual, 1, "number of metadata values for %v", k)
+				require.Equal(t, v, actual[0], "value for %v", k)
+			}
+
+			for k := range badMetadata {
+				actual := md[k]
+				require.Empty(t, actual, "metadata for bad key %v", k)
+			}
 		})
 	}
-
 }
 
 func TestGetInfoRefsProxiedToGitalyInterruptedStream(t *testing.T) {
@@ -95,10 +144,10 @@ func TestGetInfoRefsProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	apiResponse.GitalyServer.Address = gitalyAddress
 
-	ts := testAuthServer(nil, 200, apiResponse)
+	ts := testAuthServer(nil, nil, 200, apiResponse)
 	defer ts.Close()
 
 	ws := startWorkhorseServer(ts.URL)
@@ -131,31 +180,41 @@ func TestPostReceivePackProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	apiResponse.GitalyServer.Address = "unix://" + socketPath
-	ts := testAuthServer(nil, 200, apiResponse)
+	apiResponse.GitalyServer.Address = "unix:" + socketPath
+	apiResponse.GitConfigOptions = []string{"git-config-hello=world"}
+	ts := testAuthServer(nil, nil, 200, apiResponse)
 	defer ts.Close()
 
 	ws := startWorkhorseServer(ts.URL)
 	defer ws.Close()
 
+	gitProtocol := "fake Git protocol"
 	resource := "/gitlab-org/gitlab-test.git/git-receive-pack"
 	resp, body := httpPost(
 		t,
 		ws.URL+resource,
-		"application/x-git-receive-pack-request",
+		map[string]string{
+			"Content-Type": "application/x-git-receive-pack-request",
+			"Git-Protocol": gitProtocol,
+		},
 		testhelper.GitalyReceivePackResponseMock,
 	)
 
-	expectedBody := strings.Join([]string{
-		apiResponse.Repository.StorageName,
-		apiResponse.Repository.RelativePath,
-		apiResponse.GL_ID,
-		apiResponse.GL_USERNAME,
-		string(testhelper.GitalyReceivePackResponseMock),
-	}, "\000")
+	split := strings.SplitN(body, "\000", 2)
+	require.Len(t, split, 2)
+
+	gitalyRequest := &gitalypb.PostReceivePackRequest{}
+	require.NoError(t, jsonpb.UnmarshalString(split[0], gitalyRequest))
+
+	assert.Equal(t, apiResponse.Repository.StorageName, gitalyRequest.Repository.StorageName)
+	assert.Equal(t, apiResponse.Repository.RelativePath, gitalyRequest.Repository.RelativePath)
+	assert.Equal(t, apiResponse.GL_ID, gitalyRequest.GlId)
+	assert.Equal(t, apiResponse.GL_USERNAME, gitalyRequest.GlUsername)
+	assert.Equal(t, apiResponse.GitConfigOptions, gitalyRequest.GitConfigOptions)
+	assert.Equal(t, gitProtocol, gitalyRequest.GitProtocol)
 
 	assert.Equal(t, 200, resp.StatusCode, "POST %q", resource)
-	assert.Equal(t, expectedBody, body, "POST %q: response body", resource)
+	require.Equal(t, string(testhelper.GitalyReceivePackResponseMock), split[1])
 	testhelper.AssertResponseHeader(t, resp, "Content-Type", "application/x-git-receive-pack-result")
 }
 
@@ -165,8 +224,8 @@ func TestPostReceivePackProxiedToGitalyInterrupted(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	apiResponse.GitalyServer.Address = "unix://" + socketPath
-	ts := testAuthServer(nil, 200, apiResponse)
+	apiResponse.GitalyServer.Address = "unix:" + socketPath
+	ts := testAuthServer(nil, nil, 200, apiResponse)
 	defer ts.Close()
 
 	ws := startWorkhorseServer(ts.URL)
@@ -215,37 +274,45 @@ func TestPostUploadPackProxiedToGitalySuccessfully(t *testing.T) {
 			gitalyServer, socketPath := startGitalyServer(t, tc.code)
 			defer gitalyServer.Stop()
 
-			apiResponse.GitalyServer.Address = "unix://" + socketPath
-			ts := testAuthServer(nil, 200, apiResponse)
+			apiResponse.GitalyServer.Address = "unix:" + socketPath
+			ts := testAuthServer(nil, nil, 200, apiResponse)
 			defer ts.Close()
 
 			ws := startWorkhorseServer(ts.URL)
 			defer ws.Close()
 
+			gitProtocol := "fake git protocol"
 			resource := "/gitlab-org/gitlab-test.git/git-upload-pack"
 			resp, body := httpPost(
 				t,
 				ws.URL+resource,
-				"application/x-git-upload-pack-request",
+				map[string]string{
+					"Content-Type": "application/x-git-upload-pack-request",
+					"Git-Protocol": gitProtocol,
+				},
 				testhelper.GitalyUploadPackResponseMock,
 			)
 
-			expectedBodyParts := []string{
-				apiResponse.Repository.StorageName,
-				apiResponse.Repository.RelativePath,
-			}
-			if tc.showAllRefs {
-				expectedBodyParts = append(expectedBodyParts, git.GitConfigShowAllRefs+"\n")
-			} else {
-				expectedBodyParts = append(expectedBodyParts, "\n")
-			}
-
-			expectedBodyParts = append(expectedBodyParts, string(testhelper.GitalyUploadPackResponseMock))
-			expectedBody := strings.Join(expectedBodyParts, "\000")
-
-			assert.Equal(t, 200, resp.StatusCode, "POST %q", resource)
-			assert.Equal(t, expectedBody, body, "POST %q: response body", resource)
+			require.Equal(t, 200, resp.StatusCode, "POST %q", resource)
 			testhelper.AssertResponseHeader(t, resp, "Content-Type", "application/x-git-upload-pack-result")
+
+			bodySplit := strings.SplitN(body, "\000", 2)
+			require.Len(t, bodySplit, 2)
+
+			gitalyRequest := &gitalypb.PostUploadPackRequest{}
+			require.NoError(t, jsonpb.UnmarshalString(bodySplit[0], gitalyRequest))
+
+			require.Equal(t, apiResponse.Repository.StorageName, gitalyRequest.Repository.StorageName)
+			require.Equal(t, apiResponse.Repository.RelativePath, gitalyRequest.Repository.RelativePath)
+			require.Equal(t, gitProtocol, gitalyRequest.GitProtocol)
+
+			if tc.showAllRefs {
+				require.Equal(t, []string{git.GitConfigShowAllRefs}, gitalyRequest.GitConfigOptions)
+			} else {
+				require.Empty(t, gitalyRequest.GitConfigOptions)
+			}
+
+			require.Equal(t, string(testhelper.GitalyUploadPackResponseMock), bodySplit[1], "POST %q: response body", resource)
 		})
 	}
 }
@@ -256,8 +323,8 @@ func TestPostUploadPackProxiedToGitalyInterrupted(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	apiResponse.GitalyServer.Address = "unix://" + socketPath
-	ts := testAuthServer(nil, 200, apiResponse)
+	apiResponse.GitalyServer.Address = "unix:" + socketPath
+	ts := testAuthServer(nil, nil, 200, apiResponse)
 	defer ts.Close()
 
 	ws := startWorkhorseServer(ts.URL)
@@ -289,95 +356,11 @@ func TestPostUploadPackProxiedToGitalyInterrupted(t *testing.T) {
 	}
 }
 
-func TestGetInfoRefsHandledLocallyDueToEmptyGitalySocketPath(t *testing.T) {
-	gitalyServer, _ := startGitalyServer(t, codes.OK)
-	defer gitalyServer.Stop()
-
-	apiResponse := gitOkBody(t)
-	apiResponse.GitalyServer.Address = ""
-	ts := testAuthServer(nil, 200, apiResponse)
-	defer ts.Close()
-
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	resource := "/gitlab-org/gitlab-test.git/info/refs?service=git-upload-pack"
-	resp, body := httpGet(t, ws.URL+resource)
-
-	assert.Equal(t, 200, resp.StatusCode, "GET %q", resource)
-	assert.NotContains(t, string(testhelper.GitalyInfoRefsResponseMock), body, "GET %q: should not have been proxied to Gitaly", resource)
-	testhelper.AssertResponseHeader(t, resp, "Content-Type", "application/x-git-upload-pack-advertisement")
-}
-
-func TestPostReceivePackHandledLocallyDueToEmptyGitalySocketPath(t *testing.T) {
-	gitalyServer, _ := startGitalyServer(t, codes.OK)
-	defer gitalyServer.Stop()
-
-	apiResponse := gitOkBody(t)
-	apiResponse.GitalyServer.Address = ""
-	ts := testAuthServer(nil, 200, apiResponse)
-	defer ts.Close()
-
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	resource := "/gitlab-org/gitlab-test.git/git-receive-pack"
-	payload := []byte("This payload should not reach Gitaly")
-	resp, body := httpPost(t, ws.URL+resource, "application/x-git-receive-pack-request", payload)
-
-	assert.Equal(t, 200, resp.StatusCode, "POST %q: status code", resource)
-	assert.NotContains(t, payload, body, "POST %q: request should not have been proxied to Gitaly", resource)
-	testhelper.AssertResponseHeader(t, resp, "Content-Type", "application/x-git-receive-pack-result")
-}
-
-func TestPostUploadPackHandledLocallyDueToEmptyGitalySocketPath(t *testing.T) {
-	gitalyServer, _ := startGitalyServer(t, codes.OK)
-	defer gitalyServer.Stop()
-
-	apiResponse := gitOkBody(t)
-	apiResponse.GitalyServer.Address = ""
-	ts := testAuthServer(nil, 200, apiResponse)
-	defer ts.Close()
-
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	resource := "/gitlab-org/gitlab-test.git/git-upload-pack"
-	payload := []byte("This payload should not reach Gitaly")
-	resp, body := httpPost(t, ws.URL+resource, "application/x-git-upload-pack-request", payload)
-
-	assert.Equal(t, 200, resp.StatusCode, "POST %q: status code", resource)
-	assert.NotContains(t, payload, body, "POST %q: request should not have been proxied to Gitaly", resource)
-	testhelper.AssertResponseHeader(t, resp, "Content-Type", "application/x-git-upload-pack-result")
-}
-
-func TestGetBlobProxiedToGitalySuccessfully(t *testing.T) {
-	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
-	defer gitalyServer.Stop()
-
-	gitalyAddress := "unix://" + socketPath
-	repoStorage := "default"
-	oid := "54fcc214b94e78d7a41a9a8fe6d87a5e59500e51"
-	repoRelativePath := "foo/bar.git"
-	jsonParams := fmt.Sprintf(`{"GitalyServer":{"Address":"%s","Token":""},"GetBlobRequest":{"repository":{"storage_name":"%s","relative_path":"%s"},"oid":"%s","limit":-1}}`,
-		gitalyAddress, repoStorage, repoRelativePath, oid)
-	expectedBody := testhelper.GitalyGetBlobResponseMock
-	blobLength := len(expectedBody)
-
-	resp, body, err := doSendDataRequest("/something", "git-blob", jsonParams)
-	require.NoError(t, err)
-
-	assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resp.Request.URL)
-	assert.Equal(t, expectedBody, string(body), "GET %q: response body", resp.Request.URL)
-	assert.Equal(t, blobLength, len(body), "GET %q: body size", resp.Request.URL)
-	testhelper.AssertResponseHeader(t, resp, "Content-Length", strconv.Itoa(blobLength))
-}
-
 func TestGetDiffProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	rightCommit := "e395f646b1499e8e0279445fc99a0596a65fab7e"
 	leftCommit := "8a0f2ee90d940bfb0ba1e14e8214b0649056e4ab"
@@ -397,7 +380,7 @@ func TestGetPatchProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	rightCommit := "e395f646b1499e8e0279445fc99a0596a65fab7e"
 	leftCommit := "8a0f2ee90d940bfb0ba1e14e8214b0649056e4ab"
@@ -417,7 +400,7 @@ func TestGetBlobProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	oid := "54fcc214b94e78d7a41a9a8fe6d87a5e59500e51"
 	repoRelativePath := "foo/bar.git"
@@ -448,7 +431,7 @@ func TestGetArchiveProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	oid := "54fcc214b94e78d7a41a9a8fe6d87a5e59500e51"
 	repoRelativePath := "foo/bar.git"
@@ -489,7 +472,7 @@ func TestGetArchiveProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	oid := "54fcc214b94e78d7a41a9a8fe6d87a5e59500e51"
 	repoRelativePath := "foo/bar.git"
@@ -522,7 +505,7 @@ func TestGetDiffProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	rightCommit := "e395f646b1499e8e0279445fc99a0596a65fab7e"
 	leftCommit := "8a0f2ee90d940bfb0ba1e14e8214b0649056e4ab"
@@ -554,7 +537,7 @@ func TestGetPatchProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	repoStorage := "default"
 	rightCommit := "e395f646b1499e8e0279445fc99a0596a65fab7e"
 	leftCommit := "8a0f2ee90d940bfb0ba1e14e8214b0649056e4ab"
@@ -586,7 +569,7 @@ func TestGetSnapshotProxiedToGitalySuccessfully(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 	expectedBody := testhelper.GitalyGetSnapshotResponseMock
 	archiveLength := len(expectedBody)
 
@@ -608,7 +591,7 @@ func TestGetSnapshotProxiedToGitalyInterruptedStream(t *testing.T) {
 	gitalyServer, socketPath := startGitalyServer(t, codes.OK)
 	defer gitalyServer.Stop()
 
-	gitalyAddress := "unix://" + socketPath
+	gitalyAddress := "unix:" + socketPath
 
 	params := buildGetSnapshotParams(gitalyAddress, buildPbRepo("default", "foo/bar.git"))
 	resp, _, err := doSendDataRequest("/api/v4/projects/:id/snapshot", "git-snapshot", params)
@@ -631,8 +614,8 @@ func TestGetSnapshotProxiedToGitalyInterruptedStream(t *testing.T) {
 	}
 }
 
-func buildGetSnapshotParams(gitalyAddress string, repo *pb.Repository) string {
-	msg := serializedMessage("GetSnapshotRequest", &pb.GetSnapshotRequest{Repository: repo})
+func buildGetSnapshotParams(gitalyAddress string, repo *gitalypb.Repository) string {
+	msg := serializedMessage("GetSnapshotRequest", &gitalypb.GetSnapshotRequest{Repository: repo})
 	return buildGitalyRPCParams(gitalyAddress, msg)
 }
 
@@ -665,8 +648,8 @@ func buildGitalyRPCParams(gitalyAddress string, rpcArgs ...rpcArg) string {
 	return string(b)
 }
 
-func buildPbRepo(storageName, relativePath string) *pb.Repository {
-	return &pb.Repository{
+func buildPbRepo(storageName, relativePath string) *gitalypb.Repository {
+	return &gitalypb.Repository{
 		StorageName:  storageName,
 		RelativePath: relativePath,
 	}
@@ -682,6 +665,16 @@ func serializedMessage(name string, arg proto.Message) rpcArg {
 	return rpcArg{name, str}
 }
 
+func serializedProtoMessage(name string, arg proto.Message) rpcArg {
+	msg, err := proto.Marshal(arg)
+
+	if err != nil {
+		panic(err)
+	}
+
+	return rpcArg{name, base64.URLEncoding.EncodeToString(msg)}
+}
+
 type combinedServer struct {
 	*grpc.Server
 	*testhelper.GitalyTestServer
@@ -697,10 +690,10 @@ func startGitalyServer(t *testing.T, finalMessageCode codes.Code) (*combinedServ
 	require.NoError(t, err)
 
 	gitalyServer := testhelper.NewGitalyServer(finalMessageCode)
-	pb.RegisterSmartHTTPServiceServer(server, gitalyServer)
-	pb.RegisterBlobServiceServer(server, gitalyServer)
-	pb.RegisterRepositoryServiceServer(server, gitalyServer)
-	pb.RegisterDiffServiceServer(server, gitalyServer)
+	gitalypb.RegisterSmartHTTPServiceServer(server, gitalyServer)
+	gitalypb.RegisterBlobServiceServer(server, gitalyServer)
+	gitalypb.RegisterRepositoryServiceServer(server, gitalyServer)
+	gitalypb.RegisterDiffServiceServer(server, gitalyServer)
 
 	go server.Serve(listener)
 
