@@ -12,7 +12,7 @@ import (
 
 	"github.com/prometheus/client_golang/prometheus"
 
-	"gitlab.com/gitlab-org/gitaly-proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/gitaly"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/helper"
@@ -125,9 +125,6 @@ type Response struct {
 	Entry string `json:"entry"`
 	// Used to communicate channel session details
 	Channel *ChannelSettings
-	// Used to communicate terminal session details (Deprecated)
-	// Issue to remove this field https://gitlab.com/gitlab-org/gitlab-workhorse/issues/214
-	Terminal *TerminalSettings
 	// GitalyServer specifies an address and authentication token for a gitaly server we should connect to.
 	GitalyServer gitaly.Server
 	// Repository object for making gRPC requests to Gitaly.
@@ -190,7 +187,7 @@ func (api *API) newRequest(r *http.Request, suffix string) (*http.Request, error
 	authReq.Header.Del("Trailers")
 	authReq.Header.Del("Upgrade")
 
-	// Also forward the Host header, which is excluded from the Header map by the http libary.
+	// Also forward the Host header, which is excluded from the Header map by the http library.
 	// This allows the Host header received by the backend to be consistent with other
 	// requests not going through gitlab-workhorse.
 	authReq.Host = r.Host
@@ -299,7 +296,7 @@ func passResponseBack(httpResponse *http.Response, w http.ResponseWriter, r *htt
 	bytesTotal.Add(float64(responseBody.Len()))
 
 	for k, v := range httpResponse.Header {
-		// Accomodate broken clients that do case-sensitive header lookup
+		// Accommodate broken clients that do case-sensitive header lookup
 		if k == "Www-Authenticate" {
 			w.Header()["WWW-Authenticate"] = v
 		} else {
