@@ -10,10 +10,11 @@ import (
 	"github.com/prometheus/client_golang/prometheus"
 
 	"gitlab.com/gitlab-org/labkit/correlation"
+	"gitlab.com/gitlab-org/labkit/log"
+	"gitlab.com/gitlab-org/labkit/mask"
 	"gitlab.com/gitlab-org/labkit/tracing"
 
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/helper"
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/log"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/senddata"
 )
 
@@ -35,9 +36,19 @@ var rangeHeaderKeys = []string{
 	"Range",
 }
 
+// Keep cache headers from the original response, not the proxied response. The
+// original response comes from the Rails application, which should be the
+// source of truth for caching.
+var preserveHeaderKeys = map[string]bool{
+	"Cache-Control": true,
+	"Expires":       true,
+	"Date":          true, // Support for HTTP 1.0 proxies
+	"Pragma":        true, // Support for HTTP 1.0 proxies
+}
+
 // httpTransport defines a http.Transport with values
 // that are more restrictive than for http.DefaultTransport,
-// they define shorter TLS Handshake, and more agressive connection closing
+// they define shorter TLS Handshake, and more aggressive connection closing
 // to prevent the connection hanging and reduce FD usage
 var httpTransport = tracing.NewRoundTripper(correlation.NewInstrumentedRoundTripper(&http.Transport{
 	Proxy: http.ProxyFromEnvironment,
@@ -100,10 +111,10 @@ func (e *entry) Inject(w http.ResponseWriter, r *http.Request, sendData string)
 		return
 	}
 
-	log.WithFields(r.Context(), log.Fields{
-		"url":  helper.ScrubURLParams(params.URL),
+	log.WithContextFields(r.Context(), log.Fields{
+		"url":  mask.URL(params.URL),
 		"path": r.URL.Path,
-	}).Print("SendURL: sending")
+	}).Info("SendURL: sending")
 
 	if params.URL == "" {
 		sendURLRequestsInvalidData.Inc()
@@ -137,9 +148,11 @@ func (e *entry) Inject(w http.ResponseWriter, r *http.Request, sendData string)
 		return
 	}
 
-	// copy response headers and body
+	// copy response headers and body, except the headers from preserveHeaderKeys
 	for key, value := range resp.Header {
-		w.Header()[key] = value
+		if !preserveHeaderKeys[key] {
+			w.Header()[key] = value
+		}
 	}
 	w.WriteHeader(resp.StatusCode)
 
