@@ -10,16 +10,21 @@ import (
 
 	"github.com/golang/protobuf/jsonpb"
 	"github.com/golang/protobuf/proto"
-	log "github.com/sirupsen/logrus"
-	"gitlab.com/gitlab-org/gitaly-proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/labkit/log"
 	"golang.org/x/net/context"
 	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/metadata"
 	"google.golang.org/grpc/status"
 )
 
 type GitalyTestServer struct {
 	finalMessageCode codes.Code
 	sync.WaitGroup
+	LastIncomingMetadata metadata.MD
+	gitalypb.UnimplementedRepositoryServiceServer
+	gitalypb.UnimplementedBlobServiceServer
+	gitalypb.UnimplementedDiffServiceServer
 }
 
 var (
@@ -38,10 +43,10 @@ var (
 func init() {
 	var err error
 	if GitalyReceivePackResponseMock, err = ioutil.ReadFile(path.Join(RootDir(), "testdata/receive-pack-fixture.txt")); err != nil {
-		log.Fatal(err)
+		log.WithError(err).Fatal("Unable to read pack response")
 	}
 	if GitalyUploadPackResponseMock, err = ioutil.ReadFile(path.Join(RootDir(), "testdata/upload-pack-fixture.txt")); err != nil {
-		log.Fatal(err)
+		log.WithError(err).Fatal("Unable to read pack response")
 	}
 }
 
@@ -71,6 +76,11 @@ func (s *GitalyTestServer) InfoRefsUploadPack(in *gitalypb.InfoRefsRequest, stre
 		GitalyInfoRefsResponseMock,
 	}, "\000"))
 
+	s.LastIncomingMetadata = nil
+	if md, ok := metadata.FromIncomingContext(stream.Context()); ok {
+		s.LastIncomingMetadata = md
+	}
+
 	return s.sendInfoRefs(stream, data)
 }
 
