@@ -44,10 +44,12 @@ type routeOptions struct {
 }
 
 const (
-	apiPattern        = `^/api/`
-	ciAPIPattern      = `^/ci/api/`
-	gitProjectPattern = `^/([^/]+/){1,}[^/]+\.git/`
-	projectPattern    = `^/([^/]+/){1,}[^/]+/`
+	apiPattern           = `^/api/`
+	ciAPIPattern         = `^/ci/api/`
+	gitProjectPattern    = `^/([^/]+/){1,}[^/]+\.git/`
+	projectPattern       = `^/([^/]+/){1,}[^/]+/`
+	snippetUploadPattern = `^/uploads/personal_snippet`
+	userUploadPattern    = `^/uploads/user`
 )
 
 func compileRegexp(regexpStr string) *regexp.Regexp {
@@ -172,8 +174,10 @@ func (u *upstream) configureRoutes() {
 	defaultUpstream := static.ServeExisting(
 		u.URLPrefix,
 		staticpages.CacheDisabled,
-		static.DeployPage(static.ErrorPagesUnless(u.DevelopmentMode, uploadAccelerateProxy)),
+		static.DeployPage(static.ErrorPagesUnless(u.DevelopmentMode, staticpages.ErrorFormatHTML, uploadAccelerateProxy)),
 	)
+	probeUpstream := static.ErrorPagesUnless(u.DevelopmentMode, staticpages.ErrorFormatJSON, proxy)
+	healthUpstream := static.ErrorPagesUnless(u.DevelopmentMode, staticpages.ErrorFormatText, proxy)
 
 	u.Routes = []routeEntry{
 		// Git Clone
@@ -187,7 +191,7 @@ func (u *upstream) configureRoutes() {
 		route("POST", ciAPIPattern+`v1/builds/[0-9]+/artifacts\z`, contentEncodingHandler(artifacts.UploadArtifacts(api, proxy))),
 
 		// Terminal websocket
-		wsRoute(projectPattern+`environments/[0-9]+/terminal.ws\z`, channel.Handler(api)),
+		wsRoute(projectPattern+`-/environments/[0-9]+/terminal.ws\z`, channel.Handler(api)),
 		wsRoute(projectPattern+`-/jobs/[0-9]+/terminal.ws\z`, channel.Handler(api)),
 
 		// Proxy Job Services
@@ -200,6 +204,18 @@ func (u *upstream) configureRoutes() {
 		// Maven Artifact Repository
 		route("PUT", apiPattern+`v4/projects/[0-9]+/packages/maven/`, filestore.BodyUploader(api, proxy, nil)),
 
+		// Conan Artifact Repository
+		route("PUT", apiPattern+`v4/packages/conan/`, filestore.BodyUploader(api, proxy, nil)),
+
+		// NuGet Artifact Repository
+		route("PUT", apiPattern+`v4/projects/[0-9]+/packages/nuget/`, filestore.BodyUploader(api, proxy, nil)),
+
+		// We are porting API to disk acceleration
+		// we need to declare each routes until we have fixed all the routes on the rails codebase.
+		// Overall status can be seen at https://gitlab.com/groups/gitlab-org/-/epics/1802#current-status
+		route("POST", apiPattern+`v4/projects/[0-9]+/wikis/attachments\z`, uploadAccelerateProxy),
+		route("POST", apiPattern+`graphql\z`, uploadAccelerateProxy),
+
 		// Explicitly proxy API requests
 		route("", apiPattern, proxy),
 		route("", ciAPIPattern, proxy),
@@ -217,12 +233,20 @@ func (u *upstream) configureRoutes() {
 
 		// Uploads
 		route("POST", projectPattern+`uploads\z`, upload.Accelerate(api, proxy)),
+		route("POST", snippetUploadPattern, upload.Accelerate(api, proxy)),
+		route("POST", userUploadPattern, upload.Accelerate(api, proxy)),
 
 		// For legacy reasons, user uploads are stored under the document root.
 		// To prevent anybody who knows/guesses the URL of a user-uploaded file
 		// from downloading it we make sure requests to /uploads/ do _not_ pass
 		// through static.ServeExisting.
-		route("", `^/uploads/`, static.ErrorPagesUnless(u.DevelopmentMode, proxy)),
+		route("", `^/uploads/`, static.ErrorPagesUnless(u.DevelopmentMode, staticpages.ErrorFormatHTML, proxy)),
+
+		// health checks don't intercept errors and go straight to rails
+		// TODO: We should probably not return a HTML deploy page?
+		//       https://gitlab.com/gitlab-org/gitlab-workhorse/issues/230
+		route("", "^/-/(readiness|liveness)$", static.DeployPage(probeUpstream)),
+		route("", "^/-/health$", static.DeployPage(healthUpstream)),
 
 		// This route lets us filter out health checks from our metrics.
 		route("", "^/-/", defaultUpstream),
