@@ -9,28 +9,27 @@ import (
 	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
+	"net/url"
 	"os"
 	"os/exec"
 	"path"
 	"regexp"
-	"strconv"
 	"testing"
 	"time"
 
-	log "github.com/sirupsen/logrus"
+	"github.com/sirupsen/logrus"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/labkit/log"
 
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/config"
-	"gitlab.com/gitlab-org/gitlab-workhorse/internal/git"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/gitaly"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/helper"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/secret"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/testhelper"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/upstream"
-
-	pb "gitlab.com/gitlab-org/gitaly-proto/go"
-
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 const scratchDir = "testdata/scratch"
@@ -43,73 +42,17 @@ var checkoutDir = path.Join(scratchDir, "test")
 var cacheDir = path.Join(scratchDir, "cache")
 
 func TestMain(m *testing.M) {
-	git.Testing = true
-
-	source := "https://gitlab.com/gitlab-org/gitlab-test.git"
-	clonePath := path.Join(testRepoRoot, testRepo)
-	if _, err := os.Stat(clonePath); err != nil {
-		testCmd := exec.Command("git", "clone", "--bare", source, clonePath)
-		testCmd.Stdout = os.Stdout
-		testCmd.Stderr = os.Stderr
-
-		if err := testCmd.Run(); err != nil {
-			log.WithField("testCmd", testCmd).Print("Test setup: failed to run")
-			os.Exit(-1)
-		}
+	if _, err := os.Stat(path.Join(testRepoRoot, testRepo)); os.IsNotExist(err) {
+		log.WithError(err).Fatal("cannot find test repository. Please run 'make prepare-tests'")
 	}
 
-	cleanup, err := testhelper.BuildExecutables()
-	if err != nil {
-		log.WithError(err).Print("Test setup: failed to build executables")
-		os.Exit(1)
+	if err := testhelper.BuildExecutables(); err != nil {
+		log.WithError(err).Fatal()
 	}
 
 	defer gitaly.CloseConnections()
 
-	os.Exit(func() int {
-		defer cleanup()
-		return m.Run()
-	}())
-}
-
-func TestAllowedClone(t *testing.T) {
-	// Prepare clone directory
-	require.NoError(t, os.RemoveAll(scratchDir))
-
-	// Prepare test server and backend
-	ts := testAuthServer(nil, 200, gitOkBody(t))
-	defer ts.Close()
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	// Do the git clone
-	cloneCmd := exec.Command("git", "clone", fmt.Sprintf("%s/%s", ws.URL, testRepo), checkoutDir)
-	runOrFail(t, cloneCmd)
-
-	// We may have cloned an 'empty' repository, 'git log' will fail in it
-	logCmd := exec.Command("git", "log", "-1", "--oneline")
-	logCmd.Dir = checkoutDir
-	runOrFail(t, logCmd)
-}
-
-func TestAllowedShallowClone(t *testing.T) {
-	// Prepare clone directory
-	require.NoError(t, os.RemoveAll(scratchDir))
-
-	// Prepare test server and backend
-	ts := testAuthServer(nil, 200, gitOkBody(t))
-	defer ts.Close()
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	// Shallow git clone (depth 1)
-	cloneCmd := exec.Command("git", "clone", "--depth", "1", fmt.Sprintf("%s/%s", ws.URL, testRepo), checkoutDir)
-	runOrFail(t, cloneCmd)
-
-	// We may have cloned an 'empty' repository, 'git log' will fail in it
-	logCmd := exec.Command("git", "log", "-1", "--oneline")
-	logCmd.Dir = checkoutDir
-	runOrFail(t, logCmd)
+	os.Exit(m.Run())
 }
 
 func TestDeniedClone(t *testing.T) {
@@ -117,7 +60,7 @@ func TestDeniedClone(t *testing.T) {
 	require.NoError(t, os.RemoveAll(scratchDir))
 
 	// Prepare test server and backend
-	ts := testAuthServer(nil, 403, "Access denied")
+	ts := testAuthServer(nil, nil, 403, "Access denied")
 	defer ts.Close()
 	ws := startWorkhorseServer(ts.URL)
 	defer ws.Close()
@@ -129,26 +72,9 @@ func TestDeniedClone(t *testing.T) {
 	assert.Error(t, err, "git clone should have failed")
 }
 
-func TestAllowedPush(t *testing.T) {
-	preparePushRepo(t)
-
-	// Prepare the test server and backend
-	ts := testAuthServer(nil, 200, gitOkBody(t))
-	defer ts.Close()
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	// Perform the git push
-	pushCmd := exec.Command("git", "push", fmt.Sprintf("%s/%s", ws.URL, testRepo), fmt.Sprintf("master:%s", newBranch()))
-	pushCmd.Dir = checkoutDir
-	runOrFail(t, pushCmd)
-}
-
 func TestDeniedPush(t *testing.T) {
-	preparePushRepo(t)
-
 	// Prepare the test server and backend
-	ts := testAuthServer(nil, 403, "Access denied")
+	ts := testAuthServer(nil, nil, 403, "Access denied")
 	defer ts.Close()
 	ws := startWorkhorseServer(ts.URL)
 	defer ws.Close()
@@ -181,7 +107,7 @@ func TestRegularProjectsAPI(t *testing.T) {
 		"/api/v3/projects/foo%2Fbar%2Fbaz/repository/not/special",
 		"/api/v3/projects/foo%2Fbar%2Fbaz%2Fqux/repository/not/special",
 	} {
-		resp, body := httpGet(t, ws.URL+resource)
+		resp, body := httpGet(t, ws.URL+resource, nil)
 
 		assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resource)
 		assert.Equal(t, apiResponse, body, "GET %q: response body", resource)
@@ -220,7 +146,7 @@ func TestAllowedStaticFile(t *testing.T) {
 		"/static%20file.txt",
 		"/static file.txt",
 	} {
-		resp, body := httpGet(t, ws.URL+resource)
+		resp, body := httpGet(t, ws.URL+resource, nil)
 
 		assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resource)
 		assert.Equal(t, content, body, "GET %q: response body", resource)
@@ -236,12 +162,12 @@ func TestStaticFileRelativeURL(t *testing.T) {
 	ts := testhelper.TestServerWithHandler(regexp.MustCompile(`.`), http.HandlerFunc(http.NotFound))
 	defer ts.Close()
 	backendURLString := ts.URL + "/my-relative-url"
-	log.Print(backendURLString)
+	log.Info(backendURLString)
 	ws := startWorkhorseServer(backendURLString)
 	defer ws.Close()
 
 	resource := "/my-relative-url/static.txt"
-	resp, body := httpGet(t, ws.URL+resource)
+	resp, body := httpGet(t, ws.URL+resource, nil)
 
 	assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resource)
 	assert.Equal(t, content, body, "GET %q: response body", resource)
@@ -265,7 +191,7 @@ func TestAllowedPublicUploadsFile(t *testing.T) {
 		"/uploads/static%20file.txt",
 		"/uploads/static file.txt",
 	} {
-		resp, body := httpGet(t, ws.URL+resource)
+		resp, body := httpGet(t, ws.URL+resource, nil)
 
 		assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resource)
 		assert.Equal(t, content, body, "GET %q: response body", resource)
@@ -290,7 +216,7 @@ func TestDeniedPublicUploadsFile(t *testing.T) {
 		"/uploads/static.txt",
 		"/uploads%2Fstatic.txt",
 	} {
-		resp, body := httpGet(t, ws.URL+resource)
+		resp, body := httpGet(t, ws.URL+resource, nil)
 
 		assert.Equal(t, 404, resp.StatusCode, "GET %q: status code", resource)
 		assert.Equal(t, "", body, "GET %q: response body", resource)
@@ -321,7 +247,7 @@ This is a static error page for code 499
 	defer ws.Close()
 
 	resourcePath := "/error-499"
-	resp, body := httpGet(t, ws.URL+resourcePath)
+	resp, body := httpGet(t, ws.URL+resourcePath, nil)
 
 	assert.Equal(t, 499, resp.StatusCode, "GET %q: status code", resourcePath)
 	assert.Equal(t, string(errorPageBody), body, "GET %q: response body", resourcePath)
@@ -391,8 +317,6 @@ func sendDataResponder(command string, literalJSON string) *httptest.Server {
 		if _, err := fmt.Fprintf(w, "gibberish"); err != nil {
 			panic(err)
 		}
-
-		return
 	}
 
 	return testhelper.TestServerWithHandler(regexp.MustCompile(`.`), handler)
@@ -459,53 +383,6 @@ func TestSendURLForArtifacts(t *testing.T) {
 	assert.Equal(t, fileContents, string(body), "GET %q: response body", resourcePath)
 }
 
-func TestGetGitBlob(t *testing.T) {
-	blobID := "50b27c6518be44c42c4d87966ae2481ce895624c" // the LICENSE file in the test repository
-	blobLength := 1075
-	jsonParams := fmt.Sprintf(`{"RepoPath":"%s","BlobId":"%s"}`, path.Join(testRepoRoot, testRepo), blobID)
-	expectedBody := "The MIT License (MIT)"
-
-	resp, body, err := doSendDataRequest("/something", "git-blob", jsonParams)
-	require.NoError(t, err)
-
-	assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resp.Request.URL)
-	assert.Equal(t, expectedBody, string(body[:len(expectedBody)]), "GET %q: response body", resp.Request.URL)
-	assert.Equal(t, blobLength, len(body), "GET %q: body size", resp.Request.URL)
-	testhelper.AssertResponseHeader(t, resp, "Content-Length", strconv.Itoa(blobLength))
-	assertNginxResponseBuffering(t, "no", resp, "GET %q: nginx response buffering", resp.Request.URL)
-}
-
-func TestGetGitDiff(t *testing.T) {
-	fromSha := "be93687618e4b132087f430a4d8fc3a609c9b77c"
-	toSha := "54fcc214b94e78d7a41a9a8fe6d87a5e59500e51"
-	jsonParams := fmt.Sprintf(`{"RepoPath":"%s","ShaFrom":"%s","ShaTo":"%s"}`, path.Join(testRepoRoot, testRepo), fromSha, toSha)
-	expectedBody := "diff --git a/README b/README"
-
-	resp, body, err := doSendDataRequest("/something", "git-diff", jsonParams)
-	require.NoError(t, err)
-
-	assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resp.Request.URL)
-	assert.Equal(t, expectedBody, string(body[:len(expectedBody)]), "GET %q: response body", resp.Request.URL)
-	assert.Equal(t, 155, len(body), "GET %q: body size", resp.Request.URL)
-	assertNginxResponseBuffering(t, "no", resp, "GET %q: nginx response buffering", resp.Request.URL)
-}
-
-func TestGetGitPatch(t *testing.T) {
-	// HEAD of master branch against HEAD of fix branch
-	fromSha := "6907208d755b60ebeacb2e9dfea74c92c3449a1f"
-	toSha := "48f0be4bd10c1decee6fae52f9ae6d10f77b60f4"
-	jsonParams := fmt.Sprintf(`{"RepoPath":"%s","ShaFrom":"%s","ShaTo":"%s"}`, path.Join(testRepoRoot, testRepo), fromSha, toSha)
-
-	resp, body, err := doSendDataRequest("/something", "git-format-patch", jsonParams)
-	require.NoError(t, err)
-
-	assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resp.Request.URL)
-	assertNginxResponseBuffering(t, "no", resp, "GET %q: nginx response buffering", resp.Request.URL)
-
-	// Only the two commits on the fix branch should be included
-	testhelper.AssertPatchSeries(t, body, "12d65c8dd2b2676fa3ac47d955accc085a37a9c1", toSha)
-}
-
 func TestApiContentTypeBlock(t *testing.T) {
 	wrongResponse := `{"hello":"world"}`
 	ts := testhelper.TestServerWithHandler(regexp.MustCompile(`.`), func(w http.ResponseWriter, _ *http.Request) {
@@ -519,7 +396,7 @@ func TestApiContentTypeBlock(t *testing.T) {
 	defer ws.Close()
 
 	resourcePath := "/something"
-	resp, body := httpGet(t, ws.URL+resourcePath)
+	resp, body := httpGet(t, ws.URL+resourcePath, nil)
 
 	assert.Equal(t, 500, resp.StatusCode, "GET %q: status code", resourcePath)
 	assert.NotContains(t, wrongResponse, body, "GET %q: response body", resourcePath)
@@ -528,7 +405,7 @@ func TestApiContentTypeBlock(t *testing.T) {
 func TestAPIFalsePositivesAreProxied(t *testing.T) {
 	goodResponse := []byte(`<html></html>`)
 	ts := testhelper.TestServerWithHandler(regexp.MustCompile(`.`), func(w http.ResponseWriter, r *http.Request) {
-		if r.Header.Get(api.RequestHeader) != "" && r.Method != "GET" {
+		if r.Header.Get(secret.RequestHeader) != "" && r.Method != "GET" {
 			w.WriteHeader(500)
 			w.Write([]byte("non-GET request went through PreAuthorize handler"))
 		} else {
@@ -573,6 +450,26 @@ func TestAPIFalsePositivesAreProxied(t *testing.T) {
 	}
 }
 
+func TestCorrelationIdHeader(t *testing.T) {
+	ts := testhelper.TestServerWithHandler(regexp.MustCompile(`.`), func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Add("X-Request-Id", "12345678")
+		w.WriteHeader(200)
+	})
+	defer ts.Close()
+	ws := startWorkhorseServer(ts.URL)
+	defer ws.Close()
+
+	for _, resource := range []string{
+		"/api/v3/projects/123/repository/not/special",
+	} {
+		resp, _ := httpGet(t, ws.URL+resource, nil)
+
+		assert.Equal(t, 200, resp.StatusCode, "GET %q: status code", resource)
+		requestIds := resp.Header["X-Request-Id"]
+		assert.Equal(t, 1, len(requestIds), "GET %q: One X-Request-Id present", resource)
+	}
+}
+
 func setupStaticFile(fpath, content string) error {
 	cwd, err := os.Getwd()
 	if err != nil {
@@ -583,10 +480,7 @@ func setupStaticFile(fpath, content string) error {
 		return err
 	}
 	staticFile := path.Join(*documentRoot, fpath)
-	if err := ioutil.WriteFile(staticFile, []byte(content), 0666); err != nil {
-		return err
-	}
-	return nil
+	return ioutil.WriteFile(staticFile, []byte(content), 0666)
 }
 
 func prepareDownloadDir(t *testing.T) {
@@ -594,23 +488,35 @@ func prepareDownloadDir(t *testing.T) {
 	require.NoError(t, os.MkdirAll(scratchDir, 0755))
 }
 
-func preparePushRepo(t *testing.T) {
-	require.NoError(t, os.RemoveAll(scratchDir))
-	cloneCmd := exec.Command("git", "clone", path.Join(testRepoRoot, testRepo), checkoutDir)
-	runOrFail(t, cloneCmd)
-}
-
 func newBranch() string {
 	return fmt.Sprintf("branch-%d", time.Now().UnixNano())
 }
 
-func testAuthServer(url *regexp.Regexp, code int, body interface{}) *httptest.Server {
+func testAuthServer(url *regexp.Regexp, params url.Values, code int, body interface{}) *httptest.Server {
 	return testhelper.TestServerWithHandler(url, func(w http.ResponseWriter, r *http.Request) {
 		w.Header().Set("Content-Type", api.ResponseContentType)
 
+		logEntry := log.WithFields(log.Fields{
+			"method": r.Method,
+			"url":    r.URL,
+		})
+		logEntryWithCode := logEntry.WithField("code", code)
+
+		if params != nil {
+			currentParams := r.URL.Query()
+			for key := range params {
+				if currentParams.Get(key) != params.Get(key) {
+					logEntry.Info("UPSTREAM", "DENY", "invalid auth server params")
+					w.WriteHeader(http.StatusForbidden)
+					return
+				}
+			}
+		}
+
 		// Write pure string
 		if data, ok := body.(string); ok {
-			log.Println("UPSTREAM", r.Method, r.URL, code)
+			logEntryWithCode.Info("UPSTREAM")
+
 			w.WriteHeader(code)
 			fmt.Fprint(w, data)
 			return
@@ -619,39 +525,20 @@ func testAuthServer(url *regexp.Regexp, code int, body interface{}) *httptest.Se
 		// Write json string
 		data, err := json.Marshal(body)
 		if err != nil {
-			log.Println("UPSTREAM", r.Method, r.URL, "FAILURE", err)
+			logEntry.WithError(err).Error("UPSTREAM")
+
 			w.WriteHeader(503)
 			fmt.Fprint(w, err)
 			return
 		}
 
-		log.Println("UPSTREAM", r.Method, r.URL, code)
+		logEntryWithCode.Info("UPSTREAM")
+
 		w.WriteHeader(code)
 		w.Write(data)
 	})
 }
 
-func archiveOKServer(t *testing.T, archiveName string) *httptest.Server {
-	return testhelper.TestServerWithHandler(regexp.MustCompile("."), func(w http.ResponseWriter, r *http.Request) {
-		cwd, err := os.Getwd()
-		require.NoError(t, err)
-
-		archivePath := path.Join(cwd, cacheDir, archiveName)
-
-		params := struct{ RepoPath, ArchivePath, CommitID, ArchivePrefix string }{
-			repoPath(t),
-			archivePath,
-			"c7fbe50c7c7419d9701eebe64b1fdacc3df5b9dd",
-			"foobar123",
-		}
-		jsonData, err := json.Marshal(params)
-		require.NoError(t, err)
-
-		encodedJSON := base64.URLEncoding.EncodeToString(jsonData)
-		w.Header().Set("Gitlab-Workhorse-Send-Data", "git-archive:"+encodedJSON)
-	})
-}
-
 func newUpstreamConfig(authBackend string) *config.Config {
 	return &config.Config{
 		Version:      "123",
@@ -666,7 +553,7 @@ func startWorkhorseServer(authBackend string) *httptest.Server {
 
 func startWorkhorseServerWithConfig(cfg *config.Config) *httptest.Server {
 	testhelper.ConfigureSecret()
-	u := upstream.NewUpstream(*cfg)
+	u := upstream.NewUpstream(*cfg, logrus.StandardLogger())
 
 	return httptest.NewServer(u)
 }
@@ -678,27 +565,25 @@ func runOrFail(t *testing.T, cmd *exec.Cmd) {
 }
 
 func gitOkBody(t *testing.T) *api.Response {
-	repoPath := repoPath(t)
 	return &api.Response{
 		GL_ID:       "user-123",
 		GL_USERNAME: "username",
-		RepoPath:    repoPath,
-		Repository: pb.Repository{
+		Repository: gitalypb.Repository{
 			StorageName:  "default",
 			RelativePath: "foo/bar.git",
 		},
 	}
 }
 
-func repoPath(t *testing.T) string {
-	cwd, err := os.Getwd()
+func httpGet(t *testing.T, url string, headers map[string]string) (*http.Response, string) {
+	req, err := http.NewRequest("GET", url, nil)
 	require.NoError(t, err)
 
-	return path.Join(cwd, testRepoRoot, testRepo)
-}
+	for k, v := range headers {
+		req.Header.Set(k, v)
+	}
 
-func httpGet(t *testing.T, url string) (*http.Response, string) {
-	resp, err := http.Get(url)
+	resp, err := http.DefaultClient.Do(req)
 	require.NoError(t, err)
 	defer resp.Body.Close()
 
@@ -708,8 +593,15 @@ func httpGet(t *testing.T, url string) (*http.Response, string) {
 	return resp, string(b)
 }
 
-func httpPost(t *testing.T, url, contentType string, reqBody []byte) (*http.Response, string) {
-	resp, err := http.Post(url, contentType, bytes.NewReader(reqBody))
+func httpPost(t *testing.T, url string, headers map[string]string, reqBody []byte) (*http.Response, string) {
+	req, err := http.NewRequest("POST", url, bytes.NewReader(reqBody))
+	require.NoError(t, err)
+
+	for k, v := range headers {
+		req.Header.Set(k, v)
+	}
+
+	resp, err := http.DefaultClient.Do(req)
 	require.NoError(t, err)
 	defer resp.Body.Close()
 
@@ -723,3 +615,67 @@ func assertNginxResponseBuffering(t *testing.T, expected string, resp *http.Resp
 	actual := resp.Header.Get(helper.NginxResponseBufferHeader)
 	assert.Equal(t, expected, actual, msgAndArgs...)
 }
+
+// TestHealthChecksNoStaticHTML verifies that health endpoints pass errors through and don't return the static html error pages
+func TestHealthChecksNoStaticHTML(t *testing.T) {
+	apiResponse := "API RESPONSE"
+	errorPageBody := `<html>
+<body>
+This is a static error page for code 503
+</body>
+</html>
+`
+	require.NoError(t, setupStaticFile("503.html", errorPageBody))
+
+	ts := testhelper.TestServerWithHandler(regexp.MustCompile(`.`), func(w http.ResponseWriter, _ *http.Request) {
+		w.Header().Set("X-Gitlab-Custom-Error", "1")
+		w.WriteHeader(503)
+		_, err := w.Write([]byte(apiResponse))
+		require.NoError(t, err)
+	})
+	defer ts.Close()
+
+	ws := startWorkhorseServer(ts.URL)
+	defer ws.Close()
+
+	for _, resource := range []string{
+		"/-/health",
+		"/-/readiness",
+		"/-/liveness",
+	} {
+		t.Run(resource, func(t *testing.T) {
+			resp, body := httpGet(t, ws.URL+resource, nil)
+
+			assert.Equal(t, 503, resp.StatusCode, "status code")
+			assert.Equal(t, apiResponse, body, "response body")
+			assertNginxResponseBuffering(t, "", resp, "nginx response buffering")
+		})
+	}
+}
+
+// TestHealthChecksUnreachable verifies that health endpoints return the correct content-type when the upstream is down
+func TestHealthChecksUnreachable(t *testing.T) {
+	ws := startWorkhorseServer("http://127.0.0.1:99999") // This url should point to nothing for the test to be accurate (equivalent to upstream being down)
+	defer ws.Close()
+
+	testCases := []struct {
+		path         string
+		content      string
+		responseType string
+	}{
+		{path: "/-/health", content: "Bad Gateway\n", responseType: "text/plain; charset=utf-8"},
+		{path: "/-/readiness", content: "{\"error\":\"Bad Gateway\",\"status\":502}\n", responseType: "application/json; charset=utf-8"},
+		{path: "/-/liveness", content: "{\"error\":\"Bad Gateway\",\"status\":502}\n", responseType: "application/json; charset=utf-8"},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.path, func(t *testing.T) {
+			resp, body := httpGet(t, ws.URL+tc.path, nil)
+
+			assert.Equal(t, 502, resp.StatusCode, "status code")
+			assert.Equal(t, tc.responseType, resp.Header.Get("Content-Type"), "content-type")
+			assert.Equal(t, tc.content, body, "response body")
+			assertNginxResponseBuffering(t, "", resp, "nginx response buffering")
+		})
+	}
+}
